import inspect
from abc import ABCMeta

from pjml.cs.cs import CS


class MetaOperand(ABCMeta):
    def __add__(cls, other):
        return cls.__add__(cls, other)

    def __mul__(cls, other):
        return cls.__mul__(cls, other)

    def __matmul__(cls, other):
        return cls.__matmul__(cls, other)

    # Ensures resulting object will also accept operators.
    def __radd__(self, other):
        return self.__add__(other, self)

    def __rmul__(self, other):
        return self.__mul__(other, self)

    def __rmatmul__(self, other):
        return self.__matmul__(other, self)

    # I had to put it here, since I cannot create a specific metaclass for that.
    # Such attempt would lead to the diamond inheritance problem.
    def _name_impl(cls):
        return cls.__name__


class AsOperand(metaclass=MetaOperand):
    def __add__(self, other):
        from pjml.operator.select import Select

        if inspect.isclass(other):  # if is a class
            return other.cs.__class__.__add__(self, other)
        if isinstance(other, CS):  # if is mixed with cs level
            return other.__class__.__add__(self, other)
        if isinstance(other, Select):
            return Select(self, *other.components)
        elif isinstance(self, Select):
            return Select(*self.components, other)
        return Select(self, other)

    def __mul__(self, other):
        from pjml.operator.pipeline import Pipeline
        from pjml.operator.chain import Chain

        if inspect.isclass(other):  # if is a class --> go to cs level
            return other.cs.__class__.__mul__(self, other)
        if isinstance(other, CS):  # if is mixed with cs --> go to cs level
            return other.__class__.__mul__(self, other)
        if isinstance(other, (Pipeline, Chain)):
            return Pipeline(self, *other.components)
        if isinstance(self, (Pipeline, Chain)):
            return Pipeline(*self.components, other)
        return Pipeline(self, other)

    def __matmul__(self, other):  # @
        from pjml.operator.shuffle import Shuffle

        if inspect.isclass(other):  # if is a class
            return other.cs.__class__.__matmul__(self, other)
        if isinstance(other, CS):  # if is mixed with cs level
            return other.__class__.__matmul__(self, other)
        if isinstance(other, Shuffle):
            return Shuffle(self, *other.components)
        elif isinstance(self, Shuffle):
            return Shuffle(*self.components, other)
        return Shuffle(self, other)

    # Ensures resulting object will also accept operators.
    def __radd__(self, other):
        return self.__class__.__add__(other, self)

    def __rmul__(self, other):
        return self.__class__.__mul__(other, self)

    def __rmatmul__(self, other):
        return self.__class__.__matmul__(other, self)
