from functools import lru_cache
from typing import Callable, Any, Dict

import pjdata.types as t
from pjml.abs.minimalcontainer import MinimalContainer1


class Map(MinimalContainer1):
    """Execute the same component for the entire stream."""

    @lru_cache()
    def _enhancer_info(
        self, data: t.Data = None
    ) -> Dict[str, Any]:  # TODO: should _*info accept None?
        return {"enhancer": self.component.enhancer}

    @lru_cache()
    def _model_info(self, data: t.Data) -> Dict[str, Any]:
        return {"models": map(self.component.model, data.stream)}

    def _enhancer_func(self) -> Callable[[t.Data], t.Result]:
        enhancer = self.component.enhancer
        return lambda d: {"stream": map(enhancer.transform, d.stream)}

    def _model_func(self, data: t.Data) -> t.Transformation:
        component = self.component
        return lambda d: {"stream": map(component.model(data).transform, d.stream)}
