"""Module dedicated to framework testing."""
import numpy as np

from pjdata.content.data import Data
from pjml.data.evaluation.metric import Metric
from pjml.data.flow.file import File
from pjml.operator.chain import Chain
from pjml.operator.pipeline import Pipeline
from pjml.operator.select import Select
from pjml.operator.shuffle import Shuffle
from pjml.stream.expand.partition import Partition
from pjml.stream.reduce.reduce import Reduce
from pjml.stream.reduce.summ import Summ
from pjml.stream.transform.map import Map
from pjpy.modeling.supervised.classifier.dt import DT
from pjpy.modeling.supervised.classifier.svmc import SVMC
from pjpy.processing.feature.reductor.pca import PCA
from pjpy.processing.feature.scaler.minmax import MinMax


class TestArchitecture:
    """Tests for the framework architecture."""

    @staticmethod
    def pipeline_with_python_operators():
        return (
                File("tests/test_datasets/num_Iris.arff")
                * Partition()
                * Map((PCA() @ MinMax()) * (SVMC() + DT()) * Metric())
                * Summ()
                * Reduce()
        )

    @staticmethod
    def pipeline():
        return Pipeline(
            File("tests/test_datasets/num_Iris.arff"),
            Partition(),
            Map(Chain(Shuffle(PCA(), MinMax()), Select(SVMC() + DT()), Metric())),
            Summ(),
            Reduce(),
        )

    @staticmethod
    def run_pipeline(pipeline):
        np.random.seed(0)
        result = pipeline.dual_transform()

        # TODO: We should todo the unfrozen
        assert isinstance(result[0], Data)
        assert isinstance(result[0], Data)

    def test_random_search(self):
        np.random.seed(0)
        self.run_pipeline(self.pipeline_with_python_operators())

    def test_random_search_with_python_operators_(self):
        np.random.seed(0)
        self.run_pipeline(self.pipeline())
